@@ -1,7 +1,6 @@
 """
 Libreria per fare grafici di fit e tabelle LaTeX a partire da dati raccolti in un file Excel
-
-TODO: aggiungere comandi extra prima di salvere il grafico (not working)
+TODO: passare dimensioni immagine giuste (plt.rcParams["figure.figsize"] = (6.4,4.8)) provare 6.2,4,5 -- 6.24136,4.68101
 """
 __docformat__="numpy"
 
@@ -44,10 +43,6 @@ def _converti_latex(text):
     new_text=new_text.replace("[]",r"\(\left["+um_tex+r"\right]\)") #inserimento siunix dopo aver fatto l'escape dei caratteri speciali per latex
     return new_text
 
-def _do_extra(to_do):
-    code_to_execute = compile("\n".join(to_do), "<string>", "exec")
-    exec(code_to_execute)
-
 def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_latex,
     split_graphs=False,
     titoli=["",""],
@@ -66,10 +61,14 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
         "ls" : "dashed",
         "color" : "gray"
     },
-    style_fit={"fmt" : "o"},
-    style_residui={"fmt" : "o"},
+    style_fit={"fmt" : "."},
+    fit_line_color="darkorange",
+    style_residui={"fmt" : "."},
+    res_line_color="navy",
+    pad_percent=0.10,
     foglio="Foglio1",
-    aggiunte=["a=1","print(a)","print(foglio)"]
+    funzioni_modifica=[None, None],
+    fig_dimension=(6.24136,4.5)
 ):
     """
     funzione che genera:
@@ -110,8 +109,8 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
         residui (default: ["",""])
     
     xlabel, ylabel : str, optional
-        stringa da utilizzare per la generazione dei label del grafico, usare notezione LaTeX. Se ==None allora i nomi delle colonne
-        del file excel verranno utilizzati, convertendo automaticamente in LaTeX (default: None)
+        stringa da utilizzare per la generazione dei label degli assi del grafico, usare notazione LaTeX. Se None allora i nomi delle colonne
+        del file excel verranno utilizzati, convertendoli automaticamente in LaTeX ed interpretando um tra parentesi quadre (default: None)
 
     path : str, optional
         path directory di partenza (default: ".")
@@ -123,14 +122,31 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
         dizionario contenente i parametri opzionali da passare a pyplot.grid() (default: {"which" : "both", "ls" : "dashed", "color" : "gray"})
 
     style_fit, style_residui : dict, optional
-        dizionario contenente i parametri opzionali da passare a pyplot.errorbar() (default: {"fmt" : "o"})
+        dizionario contenente i parametri opzionali da passare a pyplot.errorbar() (default: {"fmt" : "."})
+
+    fit_line_color, res_line_color : str, optional
+        colori da applicare alla curva di fit e alla linea dello zero nel grafico dei residui (default: "darkorange", "navy")
+
+    pad_pecent: float, optional
+        dimensione percentuale dello spazio del grafico in più rispetto al minimo necessario a contenere i dati (default: 0.10)
 
     foglio : str, optional
         nome del foglio da cui prendere i dati (default: "Foglio1")
 
-    aggiunte : list, optional
-        lista di comandi da eseguire prima che il grafico venga salvato, usare ax1 per modificare il grafico di fit, ax1 per modificare
-        il grafico dei residui. nel caso di split_graphs=True le modifiche vengono apllicate al grafico dei residui (non usare ax1 e ax2 in questo caso)
+    funzioni_modifica : list, optional
+        lista contenete due funzioni (default: [None, None]). Entrambe devono avere 6 parametri: (fig, asse, x, y, params, data). La prima funzione verrà applicata al
+        grafico di fit, la seconda al grafico dei residui, i parametri che vengono passati sono:
+
+        - fig, asse: oggetti restituiti da pyplot.subplots() relativi al grafico che si vuole modificare
+            
+        - x, y: vettori contenenti il set di dati di cui si sta creando il grafico
+            
+        - params: vettore dei parametri ottimali di fit
+            
+        - data: panda dataframe rappresentante il foglio Excel
+
+    fig_dimensions : (float, float), optional
+        tupla contenete le dimensioni (x,y) dell'immagine creata in inches (default: (6.24136,4.5))
 
     Examples
     ----------------------------
@@ -195,26 +211,24 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
     y=[e for e in excel[variabile_y].tolist() if e == e]
     dy=[e for e in excel[sigma_y].tolist() if e == e]
 
-    print(t,y,dy)
-
     #fit
     popt, pcov = curve_fit(f_fit,t,y,sigma=dy)
     
     #scrittura file_tabella.tex
-    empty_space="&"*(len(popt)-1)+"\\\\" #linee vuote per aggiungere spazio prima e dopo in ogni riga
-    lines_prima=["\\renewcommand{\\arraystretch}{0.5}","\\begin{table}[h!]","\\centering"]
+    empty_space="&"*(len(popt)-1)+r"\\" #linee vuote per aggiungere spazio prima e dopo in ogni riga
+    lines_prima=[r"\renewcommand{\arraystretch}{0.5}",r"\begin{table}[h!]",r"\centering"]
     colonne="c"*len(popt) #colonne centrate in numero variaabile a seconda di len(popt)
-    lines_prima.append("\\begin{tabular}{"+colonne+"}")
-    lines_prima.append("\\toprule")
+    lines_prima.append(r"\begin{tabular}{"+colonne+r"}")
+    lines_prima.append(r"\toprule")
     lines_prima.append(empty_space)
-    lines_prima.append(' & '.join(titoli_p_latex)+' \\\\')#titoli
+    lines_prima.append(r" & ".join(titoli_p_latex)+r" \\")#titoli
     lines_prima.append(empty_space)
-    lines_prima.append("\\midrule")
+    lines_prima.append(r"\midrule")
     lines_prima.append(empty_space)
     string_popt=[str(i_opt) for i_opt in popt] #numeri convertiti in stringa
-    lines_prima.append(' & '.join(string_popt)+' \\\\') #valori
+    lines_prima.append(r" & ".join(string_popt)+r" \\") #valori
     lines_prima.append(empty_space)
-    lines_dopo=["\\bottomrule","\\end{tabular}","\\caption{risultati del fit}","\\end{table}","\\renewcommand{\\arraystretch}{1}"]
+    lines_dopo=[r"\bottomrule",r"\end{tabular}",r"\caption{risultati del fit}",r"\end{table}",r"\renewcommand{\arraystretch}{1}"]
     lines_prima.extend(lines_dopo)    
     
     with open(path+"\\"+file_result+"_tabella.tex", 'w') as tabella_tex:
@@ -222,85 +236,96 @@ def fit(file_name,file_result,variabile_x,variabile_y,sigma_y,f_fit,titoli_p_lat
         tabella_tex.close()
 
     #disegno grafico 
-    pad=(max(t)-min(t))/20
-    a = np.linspace(min(t)-pad if min(t)-pad>0 else 0, max(t)+pad, 1001)
+    plt.rcParams["figure.figsize"] = fig_dimension #dimensioni
 
-    plt.rcParams.update({"text.usetex" : True, 
+    pad=(max(t)-min(t))*pad_percent #aggiunta spazio
+    lim_inf=min(t)-pad if min(t)-pad>0 else 0
+    lim_sup=max(t)+pad
+
+    a1 = np.linspace(lim_inf, min(t), 1001) #separazione per tratteggio
+    a2 = np.linspace(min(t), max(t), 1001)
+    a3 = np.linspace(max(t), lim_sup, 1001)
+
+    plt.rcParams.update({"text.usetex" : True,   #LaTeX
         "font.family": "computer modern",
         "text.latex.preamble": "\n".join([ # plots will use this preamble
         r"\usepackage[utf8]{inputenc}",
         r"\usepackage[T1]{fontenc}",
         r"\usepackage{siunitx}",
+        r"\usepackage{amsmath}"
         ])})
     
+    if not xlabel:
+        xlabel=_converti_latex(variabile_x)
+
+    if not ylabel:
+        ylabel=_converti_latex(variabile_y)
+
     if split_graphs:
-        #grafico fit
-        plt.figure(file_result)
-        plt.title(titoli[0])
-        if xlabel != None:
-            plt.xlabel(xlabel)
-        else:
-            plt.xlabel(_converti_latex(variabile_x))
-        if ylabel != None:
-            plt.ylabel(ylabel)
-        else:
-            plt.ylabel(_converti_latex(variabile_y))
-        plt.xscale(xscale)
-        plt.yscale(yscale)
-        plt.grid(**griglia_fit)
-        plt.errorbar(t, y, dy, **style_fit)
-        plt.plot(a, f_fit(a, *popt)) # l'asterisco indica che popt viene espanso passando i parametri singolarmente e non come lista.
-        plt.savefig(path+'\\'+file_result+'.pdf')
-
-        #disegno grafico dei residui
-        plt.figure('Residui')
-        plt.title(titoli[1])
-        plt.xlabel(_converti_latex(variabile_x))
-        plt.ylabel(_converti_latex(variabile_y))
-        plt.grid(**griglia_residui)
-        y_res=y-f_fit(np.array(t), *popt)  # l'asterisco indica che popt viene espanso passando i parametri singolarmente e non come lista.
-        plt.errorbar(t, y_res, dy, **style_residui)
-        plt.plot(a,a-a)
-        plt.savefig(path+'\\'+file_result+'_residui.pdf')
+        fig1, ax1 = plt.subplots(1, 1)
+        fig2, ax2 = plt.subplots(1, 1)
     else:
-        #fit subplot
-        fig, (ax1, ax2) = plt.subplots(2, 1, gridspec_kw={'height_ratios': [4, 1]}, sharex=True)
-        ax1.set_title(titoli[0])
-        ax1.set_xscale(xscale)
-        ax1.set_yscale(yscale)
-        ax1.grid(**griglia_fit)
-        ax1.errorbar(t, y, dy, **style_fit)
-        ax1.plot(a, f_fit(a, *popt)) # l'asterisco indica che popt viene espanso passando i parametri singolarmente e non come lista.
-        
-        #residui subplot
-        ax2.set_title(titoli[1])
-        ax2.margins(0, 0.2)
-        ax2.set_xscale(xscale)
-        ax2.set_yscale(yscale)
-        ax2.grid(**griglia_residui)
-        y_res=y-f_fit(np.array(t), *popt)  # l'asterisco indica che popt viene espanso passando i parametri singolarmente e non come lista.
-        ax2.errorbar(t, y_res, dy, **style_residui)
-        ax2.plot(a, a-a)
-        ##adjust for x axi symmetry
-        low, high = ax2.get_ylim() 
-        bound = max(abs(low), abs(high)) # find the new limits
-        ax2.set_ylim(-bound, bound) # set new limits
+        fig1, (ax1, ax2) = plt.subplots(2, 1, gridspec_kw={'height_ratios': [4, 1]}, sharex=True)
 
-        #labels
-        if xlabel != None:
-            fig.supxlabel(xlabel, y=0.05)
-        else:
-            fig.supxlabel(_converti_latex(variabile_x), y=0.05)
-        if ylabel != None:
-            fig.supylabel(ylabel, x=0.03)
-        else:
-            fig.supylabel(_converti_latex(variabile_y), x=0.03)
-        
-        fig.tight_layout()
+    #grafico di fit
+    ##aspetto
+    ax1.set_title(titoli[0])
+    ax1.set_xscale(xscale)
+    ax1.set_yscale(yscale)
+    ax1.grid(**griglia_fit)
+    ##plot
+    ax1.errorbar(t, y, dy, **style_fit)
+    ax1.plot(a1, f_fit(a1, *popt), linestyle="dashed", color=fit_line_color) # l'asterisco indica che popt viene espanso passando i parametri singolarmente e non come lista.
+    ax1.plot(a2, f_fit(a2, *popt), color=fit_line_color)
+    ax1.plot(a3, f_fit(a3, *popt), linestyle="dashed", color=fit_line_color)
+    ##adjust limits
+    ax1.set_xlim(min(a1),max(a3))
 
-        fig.savefig(path+'\\'+file_result+'.pdf')
-        fig.canvas.manager.set_window_title("Grafico") 
+    #residui
+    ##aspetto
+    ax2.set_title(titoli[1])
+    ax2.margins(0, 0.2)
+    ax2.set_xscale(xscale)
+    ax2.set_yscale(yscale)
+    ax2.grid(**griglia_residui)
+    ##plot
+    y_res=y-f_fit(np.array(t), *popt)  # l'asterisco indica che popt viene espanso passando i parametri singolarmente e non come lista.
+    ax2.errorbar(t, y_res, dy, **style_residui)
+    ax2.plot(a1, a1-a1, color=res_line_color)
+    ax2.plot(a2, a2-a2, color=res_line_color)
+    ax2.plot(a3, a3-a3, color=res_line_color)
+    ##adjust for x axi symmetry
+    low, high = ax2.get_ylim() 
+    bound = max(abs(low), abs(high)) # find the new limits
+    ax2.set_ylim(-bound, bound) # set new limits
+    ax2.set_xlim(min(a1),max(a3))  
+
+
+    fig1.supxlabel(xlabel, y=0.05)
+    fig1.supylabel(ylabel, x=0.03)
+
+    if funzioni_modifica[0]:
+        funzioni_modifica[0](fig1, ax1,t,y,popt, excel)  
+
+    if split_graphs:
         
+        fig2.supxlabel(xlabel, y=0.05)
+        fig2.supylabel(ylabel, x=0.03)
+
+        if funzioni_modifica[1]:
+            funzioni_modifica[1](fig2, ax2,t,y,popt, excel)   
+        
+        fig2.tight_layout()
+        fig2.canvas.manager.set_window_title("Residui") 
+
+        fig2.savefig(path+'\\'+file_result+'_residui.pdf')
+    else:
+        if funzioni_modifica[1]:
+            funzioni_modifica[1](fig1, ax2,t,y,popt, excel)     
+
+    fig1.tight_layout()
+    fig1.canvas.manager.set_window_title("Grafico") 
+    fig1.savefig(path+'\\'+file_result+'.pdf')
     plt.show()
 
 def tabella(file_name,file_result,
@@ -378,7 +403,7 @@ def tabella(file_name,file_result,
         formatters=formatter,
         column_format=column_align,
         decimal=",", #separatore decimale
-        longtable=True, #tipo di tabeòòa
+        longtable=True, #tipo di tabella
         caption=table_caption,
         na_rep="", #carattere da mettere nei posti NaN
         header=my_header,
